﻿using Advantech.Motion;
using Newtonsoft.Json;
using AssistClient.Device;
using AssistClient.Utils;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Linq;
using static AssistClient.Device.AxisDev;
using static AssistClient.UI.UIAxisDev;

namespace AssistClient.UI
{
    public partial class UIAxisDev : UserControl
    {
        SynchronizationContext SyncContext = null;
         
        public Action<string> GetParamsEvent;
        //
        private AxisDevProp prop = new AxisDevProp();
        private AxisDev dev; 
        public UIAxisDev()
        {
            InitializeComponent();
            propertyGrid1.PropertyValueChanged += propertyGrid1_PropertyValueChanged;
            refreshUIState();
            //获取UI线程同步上下文
            SyncContext = SynchronizationContext.Current;
            //init(); 
        }
        private void refreshUIState()
        {
            foreach (ToolStripItem item in this.toolStrip1.Items)
            {
                if (item.Text == "打开设备")
                    item.Visible = (dev == null || !dev.IsInit);
                else
                    this.propertyGrid1.Enabled=item.Enabled = !(dev == null || !dev.IsInit);
            }
        }
        public string getParamsData()
        {
            return prop.serialize();
        }
        public bool stopNow()
        {
            if (dev != null && dev.IsInit)
            {
                dev.stopNow();
                return true;
            }
            else
                return false;
        }
        public void setParamsData(string json)
        {
            if (json == "") return;
            prop.deserialize(json);
            this.propertyGrid1.Refresh();
        }
        public void init()
        {
            this.tbtnSave.Visible = tssSave.Visible = (GetParamsEvent != null);
            this.propertyGrid1.SelectedObject = prop;
            dev = new AxisDev();
            dev.WarningEvent = (level, info) =>
            {
                txtLog.Text = $"({level}){info}";
            };

            //STATE
            dev.axisStateEvent += new System.Action<int, AxisStateType, uint>((axisIndex, type, stateValue) =>
            {
                if (axisIndex != (int)prop.AxisIndex)
                    return;

                switch (type)
                {
                    case AxisStateType.AxisState:
                        if (prop.AxState != (AxisState)stateValue)
                        {
                            prop.AxState = (AxisState)stateValue;
                            this.refreshUI();
                        }
                        break;
                    case AxisStateType.AxisIOState:
                        if (prop.AxIOStatus != stateValue)
                        {
                            prop.AxIOStatus = stateValue;

                            prop.ALM = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_ALM) > 0;//需IO close 
                            prop.EMG = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_EMG) > 0;//需重置 state 
                            prop.LMTP = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_LMTP) > 0;//右极限  true->false
                            prop.LMTN = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_LMTN) > 0;//左极限
                            prop.ORG = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_ORG) > 0;
                            prop.EZ = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_EZ) > 0;
                            prop.SVON = (stateValue & (uint)Ax_Motion_IO.AX_MOTION_IO_SVON) > 0;
                            this.refreshUI();

                        }
                        break;
                    case AxisStateType.AxisMotionState:
                        if (prop.AxMotionState != stateValue)
                        {
                            prop.AxMotionState = stateValue;
                            this.refreshUI();
                        }
                        break;
                }
            });
            //位置POS
            dev.axisPosEvent += new System.Action<int, AxisPosType, double>((axisIndex, type, pos) =>
            {
                if (axisIndex != (int)prop.AxisIndex)
                    return;

                switch (type)
                {
                    case AxisPosType.CmdPos:
                        if (prop.CmdPos != pos)
                        {
                            prop.CmdPos = pos;
                            this.refreshUI();
                        }
                        break;
                    case AxisPosType.ActualPos:
                        if (prop.ActualPos != pos)
                        {
                            prop.ActualPos = pos;
                            this.refreshUI();
                        }
                        break;
                }
            });

            if (!dev.start(Config.Axis_PulseOutMode,true))
            {
                this.closeDev();
                return;
            }
            prop.DeviceNum = dev.DeviceNum.ToString();
            refreshAxisVelParam();
            refreshUIState(); 
        }
        private void refreshUI()
        {
            SyncContext.Post(m =>
            {
                var result = m as string;
                propertyGrid1.Refresh();
                tbtnJogOnOff.Text = (prop.AxState == AxisState.STA_AX_EXT_JOG) ? "关闭Jog" : "开启Jog";
                tbtnLeft.Enabled = tbtnRight.Enabled = (prop.AxState == AxisState.STA_AX_EXT_JOG);
            }, "异步操作完成结果"); 
        }
        private void refreshAxisVelParam()
        {
            int axisIndex = (int)prop.AxisIndex; 
            //prop.HomeMode = (AxitHomeMode)Config.Axis_HomeMode[axisIndex];

            var values = dev.getAxisVelParam(axisIndex);
            prop.VelLow = dev.tomm(values[0], Config.Axis_MM2PulseNum[axisIndex]);
            prop.VelHigh = dev.tomm(values[1], Config.Axis_MM2PulseNum[axisIndex]);
            prop.Acc = dev.tomm(values[2], Config.Axis_MM2PulseNum[axisIndex]);
            prop.Dec = dev.tomm(values[3], Config.Axis_MM2PulseNum[axisIndex]);
        }
        private void closeDev()
        {
            try
            {
                if (dev == null) 
                    return; 
                dev.stop();
            }
            catch (Exception ex)
            {
                this.refreshUIState();
                MessageBox.Show(ex.Message, "失败", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        protected override void OnHandleDestroyed(EventArgs e) 
        {
            base.OnHandleDestroyed(e);
            // 在此添加需要手动释放资源的代码
            this.closeDev();
        }

        private void tbtnRun_Click(object sender, EventArgs e)
        {
            try
            { 
                dev.setAxisVelParam(prop.VelLow, prop.VelHigh, prop.Acc, prop.Dec, (int)prop.AxisIndex);
                dev.move_ptp((int)prop.AxisIndex, prop.Value, prop.MoveMode);
                //running...
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.Message, "失败", MessageBoxButtons.OK, MessageBoxIcon.Error);
            } 
        }

        private void tbtnExport_Click(object sender, EventArgs e)
        {
            string filePath = FileUtil.saveAsFile($"Axis{(int)prop.AxisIndex}配置.json", "JSON|*.json");
            if (filePath != "")
            {
                string jsonText = prop.serialize();
                File.WriteAllText(filePath, jsonText);
                MessageBox.Show("保存成功！", "成功", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void tbtnImport_Click(object sender, EventArgs e)
        {
            string filePath = FileUtil.openFile("JSON|*.json");
            if (filePath != "")
            {
                string jsonText = File.ReadAllText(filePath);
                 prop.deserialize(jsonText);
                this.propertyGrid1.Refresh();
                //this.propertyGrid1.SelectedObject= axisProp;
                //MessageBox.Show("加载成功！", "成功", MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
        }

        private void tbtnHome_Click(object sender, EventArgs e)
        {
            int axitIndex = (int)prop.AxisIndex;
            dev.setAxisVelParam(prop.VelLow, prop.VelHigh, prop.Acc, prop.Dec, (int)prop.AxisIndex);
            //dev.setAxisVelParam(prop.VelLow, prop.VelHigh, prop.Acc, prop.Dec, (int)prop.AxisIndex,true);
            dev.home(axitIndex, (uint)Config.Axis_HomeMode[axitIndex], (uint)Config.Axis_HomeDir[axitIndex]);
            //if (prop.DeviceType == AxisType.默认)
            //{
            //    dev.home((int)prop.AxisIndex);
            //}
            //else //直线电机 
            //{
            //    //dev.move_ptp((int)prop.AxisIndex, - prop.ActualPos, false);
            //    dev.move_ptp((int)prop.AxisIndex, 0, true);
            //}
        }

        private void tbtnLeft_MouseDown(object sender, MouseEventArgs e)
        {
            dev.jog((int)prop.AxisIndex, 1);
        } 
        private void tbtnLeft_MouseUp(object sender, MouseEventArgs e)
        {
            dev.stopDec((int)prop.AxisIndex);
        }

        private void tbtnRight_MouseDown(object sender, MouseEventArgs e)
        {
            dev.jog((int)prop.AxisIndex, 0);
        } 
        private void tbtnRight_MouseUp(object sender, MouseEventArgs e)
        {
            dev.stopDec((int)prop.AxisIndex);
        }

       

        private void tbtnJogOnOff_Click(object sender, EventArgs e)
        {
            if(prop.AxState==AxisState.STA_AX_EXT_JOG)
            {
                dev.closeJogMode((int)prop.AxisIndex);
                //tbtnJogOnOff.Text = "开启Jog";
            }
            else
            { 
                dev.setAxisVelParam(prop.VelLow, prop.VelHigh, prop.Acc, prop.Dec, (int)prop.AxisIndex);
                dev.openJogMode((int)prop.AxisIndex);
                //tbtnJogOnOff.Text = "关闭Jog";
            }
        }

        private void tbtnStop_Click(object sender, EventArgs e)
        {
            dev.stopDec((int)prop.AxisIndex);
        }

        private void tbtnRestState_Click(object sender, EventArgs e)
        {
            dev.resetAxisState((int)prop.AxisIndex);
        }

        private void tbtnResetPos_Click(object sender, EventArgs e)
        {
            dev.resetCmdPosition((int)prop.AxisIndex);
            dev.resetActualPosition((int)prop.AxisIndex); 
        }

        private void propertyGrid1_PropertyValueChanged(object s, PropertyValueChangedEventArgs e)
        {
            //其中包含了两个重要的属性：OldValue和ChangeItem。
            //ChangeItem是“PropertyDescriptorGridEntry”类型。一般可以通过ChangeItem.PropertyDescriptor.ComponentType查找到属性的实际类型。 
            //而通过ChangeItem的Label，则可以查看到当前在属性编辑输入框中显示的值。  

            string className = e.ChangedItem.PropertyDescriptor.ComponentType.Name;
            string propertyName = e.ChangedItem.PropertyDescriptor.Name;
            var oldValue = e.OldValue;
            var newValue = e.ChangedItem.Value;
            switch (propertyName)
            {
                case "AxisIndex":
                    refreshAxisVelParam();
                    break;
            }
            
        }

        private void tbtnSave_Click(object sender, EventArgs e)
        {
            GetParamsEvent?.Invoke(prop.serialize());
        }

        private void tsbtnOpenDev_Click(object sender, EventArgs e)
        {
            this.init();
        }
         
    }
}
