﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;
using Advantech.Motion;
using Newtonsoft.Json;
using Newtonsoft.Json.Serialization;
using ProductionControl.UI.PropExtend;
using static ProductionControl.Device.ScannerDev;
using static ProductionControl.UI.UIAxisDev;

namespace ProductionControl.UI
{
    //[TypeConverter(typeof(PropertySorter))] //此属性使类无法序列化和反序列化
    public class ScannerDevProp
    {
        public ScannerDevProp(ScannerType devType) {
            DeviceType=devType;
        }

        [PropertyOrder(1), Browsable(true), Category("1 设备"), DisplayName("设备类型"), Description("设备类型"), ReadOnly(true)]
        public ScannerType DeviceType { get;set; }
        [PropertyOrder(1), Browsable(true), Category("2 图像"), DisplayName("图像名称"), Description("图像名称"), ReadOnly(true), JsonIgnore]
        public string[] ImageList { get; set; }
        [PropertyOrder(1), Browsable(true), Category("2 图像"), DisplayName("AI引擎库"), Description("此照片所调用的AI库")]
        public AIEngineLibEnum AIEngineLib { get; set; }

        //100-9999500
        [PropertyOrder(11), Browsable(true), Category("3 参数"), DisplayName("曝光"), Description("曝光时长(us)")]
        public float ExposureTime { get; set; }
        [PropertyOrder(12), Browsable(true), Category("3 参数"), DisplayName("增益"), Description("增益")]
        public float Gain { get; set; }
        [PropertyOrder(13), Browsable(true), Category("3 参数"), DisplayName("帧率"), Description("帧率")]
        public float ResultingFrameRate { get; set; }

        [PropertyOrder(25), Browsable(true), Category("4 控制"), DisplayName("4.1 运行前Sleep(ms)"), Description("休息时间")]
        public uint SleepPre { get; set; }
        [PropertyOrder(26), Browsable(true), Category("4 控制"), DisplayName("4.2 运行后Sleep(ms)"), Description("休息时间")]
        public uint SleepLater { get; set; } 
        [PropertyOrder(27), Browsable(true), Category("4 控制"), DisplayName("4.4 禁用工序"), Description("禁用本工序(True-是；False-否)")]
        public bool Disable { get; set; }

        /// <summary>
        /// 序列化
        /// </summary>
        /// <param name="obj"></param>
        /// <returns></returns>
        public string serialize()
        {  
            return JsonConvert.SerializeObject(this);
        }
        /// <summary>
        /// 反序列化
        /// </summary>
        /// <param name="json"></param>
        /// <returns></returns>
        public void deserialize(string json)
        {
            var o=  JsonConvert.DeserializeObject<ScannerDevProp>(json);
            Type type = o.GetType();
            System.Reflection.PropertyInfo[] properties = type.GetProperties(); 
            foreach (System.Reflection.PropertyInfo property in properties)
            {
                string name = property.Name;
                if (!type.GetProperty(name).IsDefined(typeof(JsonIgnoreAttribute), true))
                {
                    var value = property.GetValue(o);
                    this.GetType().GetProperty(name).SetValue(this, value);
                }
            }  
        }
    }
     

}
